@@ -32,7 +32,7 @@ from bson.max_key import MaxKey
 from bson.min_key import MinKey
 from bson.objectid import ObjectId
 from bson.py3compat import b, binary_type
-from bson.son import SON
+from bson.son import SON, RE_TYPE
 from bson.timestamp import Timestamp
 from bson.tz_util import utc
 
@@ -52,9 +52,6 @@ except ImportError:
 PY3 = sys.version_info[0] == 3
 
 
-# This sort of sucks, but seems to be as good as it gets...
-RE_TYPE = type(re.compile(""))
-
 MAX_INT32 = 2147483647
 MIN_INT32 = -2147483648
 MAX_INT64 = 9223372036854775807
@@ -142,18 +139,24 @@ def _get_number(data, position, as_class, tz_aware, uuid_subtype):
 
 
 def _get_string(data, position, as_class, tz_aware, uuid_subtype):
-    length = struct.unpack("<i", data[position:position + 4])[0] - 1
+    length = struct.unpack("<i", data[position:position + 4])[0]
+    if (len(data) - position - 4) < length:
+        raise InvalidBSON("invalid string length")
     position += 4
-    return _get_c_string(data, position, length)
+    if data[position + length - 1:position + length] != ZERO:
+        raise InvalidBSON("invalid end of string")
+    return _get_c_string(data, position, length - 1)
 
 
 def _get_object(data, position, as_class, tz_aware, uuid_subtype):
     obj_size = struct.unpack("<i", data[position:position + 4])[0]
+    if data[position + obj_size - 1:position + obj_size] != ZERO:
+        raise InvalidBSON("bad eoo")
     encoded = data[position + 4:position + obj_size - 1]
     object = _elements_to_dict(encoded, as_class, tz_aware, uuid_subtype)
     position += obj_size
     if "$ref" in object:
-        return (DBRef(object.pop("$ref"), object.pop("$id"),
+        return (DBRef(object.pop("$ref"), object.pop("$id", None),
                       object.pop("$db", None), object), position)
     return object, position
 
@@ -217,11 +220,15 @@ def _get_boolean(data, position, as_class, tz_aware, uuid_subtype):
 
 
 def _get_date(data, position, as_class, tz_aware, uuid_subtype):
-    seconds = float(struct.unpack("<q", data[position:position + 8])[0]) / 1000.0
+    millis = struct.unpack("<q", data[position:position + 8])[0]
+    diff = millis % 1000
+    seconds = (millis - diff) / 1000
     position += 8
     if tz_aware:
-        return EPOCH_AWARE + datetime.timedelta(seconds=seconds), position
-    return EPOCH_NAIVE + datetime.timedelta(seconds=seconds), position
+        dt = EPOCH_AWARE + datetime.timedelta(seconds=seconds)
+    else:
+        dt = EPOCH_NAIVE + datetime.timedelta(seconds=seconds)
+    return dt.replace(microsecond=diff * 1000), position
 
 
 def _get_code(data, position, as_class, tz_aware, uuid_subtype):
