@@ -32,14 +32,14 @@ Example usage (serialization)::
    ...        {'bar': {'hello': 'world'}},
    ...        {'code': Code("function x() { return 1; }")},
    ...        {'bin': Binary("\x00\x01\x02\x03\x04")}])
-   '[{"foo": [1, 2]}, {"bar": {"hello": "world"}}, {"code": {"$scope": {}, "$code": "function x() { return 1; }"}}, {"bin": {"$type": 0, "$binary": "AAECAwQ=\\n"}}]'
+   '[{"foo": [1, 2]}, {"bar": {"hello": "world"}}, {"code": {"$scope": {}, "$code": "function x() { return 1; }"}}, {"bin": {"$type": "00", "$binary": "AAECAwQ="}}]'
 
 Example usage (deserialization)::
 
 .. doctest::
 
    >>> from bson.json_util import loads
-   >>> loads('[{"foo": [1, 2]}, {"bar": {"hello": "world"}}, {"code": {"$scope": {}, "$code": "function x() { return 1; }"}}, {"bin": {"$type": 0, "$binary": "AAECAwQ=\\n"}}]')
+   >>> loads('[{"foo": [1, 2]}, {"bar": {"hello": "world"}}, {"code": {"$scope": {}, "$code": "function x() { return 1; }"}}, {"bin": {"$type": "00", "$binary": "AAECAwQ="}}]')
    [{u'foo': [1, 2]}, {u'bar': {u'hello': u'world'}}, {u'code': Code('function x() { return 1; }', {})}, {u'bin': Binary('\x00\x01\x02\x03\x04', 0)}]
 
 Alternatively, you can manually pass the `default` to :func:`json.dumps`.
@@ -95,10 +95,20 @@ from bson.timestamp import Timestamp
 from bson.py3compat import PY3, binary_type, string_types
 
 
+_RE_OPT_TABLE = {
+    "i": re.I,
+    "l": re.L,
+    "m": re.M,
+    "s": re.S,
+    "u": re.U,
+    "x": re.X,
+}
+
+
 def dumps(obj, *args, **kwargs):
     """Helper function that wraps :class:`json.dumps`.
 
-    Recursive function that handles all BSON types incuding
+    Recursive function that handles all BSON types including
     :class:`~bson.binary.Binary` and :class:`~bson.code.Code`.
     """
     if not json_lib:
@@ -141,17 +151,21 @@ def object_hook(dct):
         return EPOCH_AWARE + datetime.timedelta(seconds=secs)
     if "$regex" in dct:
         flags = 0
-        if "i" in dct["$options"]:
-            flags |= re.IGNORECASE
-        if "m" in dct["$options"]:
-            flags |= re.MULTILINE
+        # PyMongo always adds $options but some other tools may not.
+        for opt in dct.get("$options", ""):
+            flags |= _RE_OPT_TABLE.get(opt, 0)
         return re.compile(dct["$regex"], flags)
     if "$minKey" in dct:
         return MinKey()
     if "$maxKey" in dct:
         return MaxKey()
     if "$binary" in dct:
-        return Binary(base64.b64decode(dct["$binary"].encode()), dct["$type"])
+        if isinstance(dct["$type"], int):
+            dct["$type"] = "%02x" % dct["$type"]
+        subtype = int(dct["$type"], 16)
+        if subtype >= 0xffffff80:  # Handle mongoexport values
+            subtype = int(dct["$type"][6:], 16)
+        return Binary(base64.b64decode(dct["$binary"].encode()), subtype)
     if "$code" in dct:
         return Code(dct["$code"], dct.get("$scope"))
     if bson.has_uuid() and "$uuid" in dct:
@@ -175,8 +189,16 @@ def default(obj):
         flags = ""
         if obj.flags & re.IGNORECASE:
             flags += "i"
+        if obj.flags & re.LOCALE:
+            flags += "l"
         if obj.flags & re.MULTILINE:
             flags += "m"
+        if obj.flags & re.DOTALL:
+            flags += "s"
+        if obj.flags & re.UNICODE:
+            flags += "u"
+        if obj.flags & re.VERBOSE:
+            flags += "x"
         return {"$regex": obj.pattern,
                 "$options": flags}
     if isinstance(obj, MinKey):
@@ -189,10 +211,10 @@ def default(obj):
         return {'$code': "%s" % obj, '$scope': obj.scope}
     if isinstance(obj, Binary):
         return {'$binary': base64.b64encode(obj).decode(),
-                '$type': obj.subtype}
+                '$type': "%02x" % obj.subtype}
     if PY3 and isinstance(obj, binary_type):
         return {'$binary': base64.b64encode(obj).decode(),
-                '$type': 0}
+                '$type': "00"}
     if bson.has_uuid() and isinstance(obj, bson.uuid.UUID):
         return {"$uuid": obj.hex}
     raise TypeError("%r is not JSON serializable" % obj)
