@@ -14,12 +14,25 @@
 
 
 """Functions and classes common to multiple pymongo modules."""
+import sys
 import warnings
 from pymongo import read_preferences
 
+from pymongo.auth import MECHANISMS
 from pymongo.read_preferences import ReadPreference
 from pymongo.errors import ConfigurationError
 
+HAS_SSL = True
+try:
+    import ssl
+except ImportError:
+    HAS_SSL = False
+
+
+# Jython 2.7 includes an incomplete ssl module. See PYTHON-498.
+if sys.platform.startswith('java'):
+    HAS_SSL = False
+
 
 def raise_config_error(key, dummy):
     """Raise ConfigurationError with the given key name."""
@@ -62,6 +75,33 @@ def validate_positive_integer(option, value):
     return val
 
 
+def validate_readable(option, value):
+    """Validates that 'value' is file-like and readable.
+    """
+    # First make sure its a string py3.3 open(True, 'r') succeeds
+    # Used in ssl cert checking due to poor ssl module error reporting
+    value = validate_basestring(option, value)
+    open(value, 'r').close()
+    return value
+
+
+def validate_cert_reqs(option, value):
+    """Validate the cert reqs are valid. It must be None or one of the three
+    values ``ssl.CERT_NONE``, ``ssl.CERT_OPTIONAL`` or ``ssl.CERT_REQUIRED``"""
+    if value is None:
+        return value
+    if HAS_SSL:
+        if value in (ssl.CERT_NONE, ssl.CERT_OPTIONAL, ssl.CERT_REQUIRED):
+            return value
+        raise ConfigurationError("The value of %s must be one of: "
+                                 "`ssl.CERT_NONE`, `ssl.CERT_OPTIONAL` or "
+                                 "`ssl.CERT_REQUIRED" % (option,))
+    else:
+        raise ConfigurationError("The value of %s is set but can't be "
+                                 "validated. The ssl module is not available"
+                                 % (option,))
+
+
 def validate_positive_integer_or_none(option, value):
     """Validate that 'value' is a positive integer or None.
     """
@@ -154,6 +194,15 @@ def validate_tag_sets(dummy, value):
     return value
 
 
+def validate_auth_mechanism(option, value):
+    """Validate the authMechanism URI option.
+    """
+    if value not in MECHANISMS:
+        raise ConfigurationError("%s must be in "
+                                 "%s" % (option, MECHANISMS))
+    return value
+
+
 # jounal is an alias for j,
 # wtimeoutms is an alias for wtimeout
 VALIDATORS = {
@@ -169,7 +218,13 @@ VALIDATORS = {
     'journal': validate_boolean,
     'connecttimeoutms': validate_timeout_or_none,
     'sockettimeoutms': validate_timeout_or_none,
+    'waitqueuetimeoutms': validate_timeout_or_none,
+    'waitqueuemultiple': validate_positive_integer_or_none,
     'ssl': validate_boolean,
+    'ssl_keyfile': validate_readable,
+    'ssl_certfile': validate_readable,
+    'ssl_cert_reqs': validate_cert_reqs,
+    'ssl_ca_certs': validate_readable,
     'readpreference': validate_read_preference,
     'read_preference': validate_read_preference,
     'tag_sets': validate_tag_sets,
@@ -177,9 +232,25 @@ VALIDATORS = {
     'secondary_acceptable_latency_ms': validate_positive_float,
     'auto_start_request': validate_boolean,
     'use_greenlets': validate_boolean,
+    'authmechanism': validate_auth_mechanism,
+    'authsource': validate_basestring,
+    'gssapiservicename': validate_basestring,
 }
 
 
+_AUTH_OPTIONS = frozenset(['gssapiservicename'])
+
+
+def validate_auth_option(option, value):
+    """Validate optional authentication parameters.
+    """
+    lower, value = validate(option, value)
+    if lower not in _AUTH_OPTIONS:
+        raise ConfigurationError('Unknown '
+                                 'authentication option: %s' % (option,))
+    return lower, value
+
+
 def validate(option, value):
     """Generic validation function.
     """
@@ -339,6 +410,14 @@ class BaseObject(object):
 
         .. note:: Accessing :attr:`write_concern` returns its value
            (a subclass of :class:`dict`), not a copy.
+
+        .. warning:: If you are using :class:`~pymongo.connection.Connection`
+           or :class:`~pymongo.replica_set_connection.ReplicaSetConnection`
+           make sure you explicitly set ``w`` to 1 (or a greater value) or
+           :attr:`safe` to ``True``. Unlike calling
+           :meth:`set_lasterror_options`, setting an option in
+           :attr:`write_concern` does not implicitly set :attr:`safe`
+           to ``True``.
         """
         # To support dict style access we have to return the actual
         # WriteConcern here, not a copy.
@@ -358,7 +437,8 @@ class BaseObject(object):
     def __set_slave_okay(self, value):
         """Property setter for slave_okay"""
         warnings.warn("slave_okay is deprecated. Please use "
-                      "read_preference instead.", DeprecationWarning)
+                      "read_preference instead.", DeprecationWarning,
+                      stacklevel=2)
         self.__slave_okay = validate_boolean('slave_okay', value)
 
     slave_okay = property(__get_slave_okay, __set_slave_okay)
@@ -386,6 +466,12 @@ class BaseObject(object):
         See :class:`~pymongo.read_preferences.ReadPreference`.
 
         .. versionadded:: 2.3
+
+        .. note:: ``secondary_acceptable_latency_ms`` is ignored when talking to a
+          replica set *through* a mongos. The equivalent is the localThreshold_ command
+          line option.
+
+        .. _localThreshold: http://docs.mongodb.org/manual/reference/mongos/#cmdoption-mongos--localThreshold
         """
         return self.__secondary_acceptable_latency_ms
 
@@ -432,7 +518,7 @@ class BaseObject(object):
         """Property setter for safe"""
         warnings.warn("safe is deprecated. Please use the"
                       " 'w' write_concern option instead.",
-                      DeprecationWarning)
+                      DeprecationWarning, stacklevel=2)
         self.__safe = validate_boolean('safe', value)
 
     safe = property(__get_safe, __set_safe)
@@ -447,7 +533,8 @@ class BaseObject(object):
         .. versionadded:: 2.0
         """
         warnings.warn("get_lasterror_options is deprecated. Please use "
-                      "write_concern instead.", DeprecationWarning)
+                      "write_concern instead.", DeprecationWarning,
+                      stacklevel=2)
         return self.__write_concern.copy()
 
     def set_lasterror_options(self, **kwargs):
@@ -467,7 +554,8 @@ class BaseObject(object):
         .. versionadded:: 2.0
         """
         warnings.warn("set_lasterror_options is deprecated. Please use "
-                      "write_concern instead.", DeprecationWarning)
+                      "write_concern instead.", DeprecationWarning,
+                      stacklevel=2)
         for key, value in kwargs.iteritems():
             self.__set_safe_option(key, value)
 
@@ -487,7 +575,8 @@ class BaseObject(object):
         .. versionadded:: 2.0
         """
         warnings.warn("unset_lasterror_options is deprecated. Please use "
-                      "write_concern instead.", DeprecationWarning)
+                      "write_concern instead.", DeprecationWarning,
+                      stacklevel=2)
         if len(options):
             for option in options:
                 self.__write_concern.pop(option, None)
@@ -526,7 +615,8 @@ class BaseObject(object):
 
         if safe is not None:
             warnings.warn("The safe parameter is deprecated. Please use "
-                          "write concern options instead.", DeprecationWarning)
+                          "write concern options instead.", DeprecationWarning,
+                          stacklevel=3)
             validate_boolean('safe', safe)
 
         # Passed options override collection level defaults.
@@ -548,7 +638,9 @@ class BaseObject(object):
 
         # Fall back to collection level defaults.
         # w=0 takes precedence over self.safe = True
-        if self.safe and self.__write_concern.get('w') != 0:
+        if self.__write_concern.get('w') == 0:
+            return False, {}
+        elif self.safe or self.__write_concern.get('w', 0) != 0:
             return True, pop1(self.__write_concern.copy())
 
         return False, {}
