@@ -14,13 +14,11 @@
 
 """Database level operations."""
 
-import warnings
-
 from bson.binary import OLD_UUID_SUBTYPE
 from bson.code import Code
 from bson.dbref import DBRef
 from bson.son import SON
-from pymongo import common, helpers
+from pymongo import auth, common, helpers
 from pymongo.collection import Collection
 from pymongo.errors import (CollectionInvalid,
                             InvalidName,
@@ -54,8 +52,7 @@ class Database(common.BaseObject):
         database name.
 
         :Parameters:
-          - `connection`: a :class:`~pymongo.connection.Connection`
-            instance
+          - `connection`: a client instance
           - `name`: database name
 
         .. mongodoc:: databases
@@ -73,7 +70,8 @@ class Database(common.BaseObject):
             raise TypeError("name must be an instance "
                             "of %s" % (basestring.__name__,))
 
-        _check_name(name)
+        if name != '$external':
+            _check_name(name)
 
         self.__name = unicode(name)
         self.__connection = connection
@@ -119,8 +117,7 @@ class Database(common.BaseObject):
 
     @property
     def connection(self):
-        """The :class:`~pymongo.connection.Connection` instance for this
-        :class:`Database`.
+        """The client instance for this :class:`Database`.
 
         .. versionchanged:: 1.3
            ``connection`` is now a property rather than a method.
@@ -222,9 +219,8 @@ class Database(common.BaseObject):
         method. Any of the following options are valid:
 
           - "size": desired initial size for the collection (in
-            bytes). must be less than or equal to 10000000000. For
-            capped collections this size is the max size of the
-            collection.
+            bytes). For capped collections this size is the max
+            size of the collection.
           - "capped": if True, this is a capped collection
           - "max": maximum number of objects if capped (optional)
 
@@ -332,9 +328,13 @@ class Database(common.BaseObject):
           - `secondary_acceptable_latency_ms`: Any replica-set member whose
             ping time is within secondary_acceptable_latency_ms of the nearest
             member may accept reads. Default 15 milliseconds.
+            **Ignored by mongos** and must be configured on the command line.
+            See the localThreshold_ option for more information.
           - `**kwargs` (optional): additional keyword arguments will
             be added to the command document before it is sent
 
+        .. note:: ``command`` ignores the ``network_timeout`` parameter.
+
         .. versionchanged:: 2.3
            Added `tag_sets` and `secondary_acceptable_latency_ms` options.
         .. versionchanged:: 2.2
@@ -348,6 +348,7 @@ class Database(common.BaseObject):
         .. versionadded:: 1.4
 
         .. mongodoc:: commands
+        .. _localThreshold: http://docs.mongodb.org/manual/reference/mongos/#cmdoption-mongos--localThreshold
         """
 
         if isinstance(command, basestring):
@@ -396,14 +397,19 @@ class Database(common.BaseObject):
 
         return result
 
-    def collection_names(self):
+    def collection_names(self, include_system_collections=True):
         """Get a list of all the collection names in this database.
+
+        :Parameters:
+          - `include_system_collections` (optional): if ``False`` list
+            will not include system collections (e.g ``system.indexes``)
         """
         results = self["system.namespaces"].find(_must_use_master=True)
         names = [r["name"] for r in results]
         names = [n[len(self.__name) + 1:] for n in names
-                 if n.startswith(self.__name + ".")]
-        names = [n for n in names if "$" not in n]
+                 if n.startswith(self.__name + ".") and "$" not in n]
+        if not include_system_collections:
+            names = [n for n in names if not n.startswith("system.")]
         return names
 
     def drop_collection(self, name_or_collection):
@@ -471,7 +477,7 @@ class Database(common.BaseObject):
                 raise CollectionInvalid("%s invalid: %s" % (name, info))
         # Sharded results
         elif "raw" in result:
-            for repl, res in result["raw"].iteritems():
+            for _, res in result["raw"].iteritems():
                 if "result" in res:
                     info = res["result"]
                     if (info.find("exception") != -1 or
@@ -608,7 +614,7 @@ class Database(common.BaseObject):
     def next(self):
         raise TypeError("'Database' object is not iterable")
 
-    def add_user(self, name, password, read_only=False):
+    def add_user(self, name, password=None, read_only=None, **kwargs):
         """Create user `name` with password `password`.
 
         Add a new user with permissions for this :class:`Database`.
@@ -617,8 +623,19 @@ class Database(common.BaseObject):
 
         :Parameters:
           - `name`: the name of the user to create
-          - `password`: the password of the user to create
-          - `read_only` (optional): if ``True`` it will make user read only
+          - `password` (optional): the password of the user to create. Can not
+            be used with the ``userSource`` argument.
+          - `read_only` (optional): if ``True`` the user will be read only
+          - `**kwargs` (optional): optional fields for the user document
+            (e.g. ``userSource``, ``otherDBRoles``, or ``roles``). See
+            `<http://docs.mongodb.org/manual/reference/privilege-documents>`_
+            for more information.
+
+        .. note:: The use of optional keyword arguments like ``userSource``,
+           ``otherDBRoles``, or ``roles`` requires MongoDB >= 2.4.0
+
+        .. versionchanged:: 2.5
+           Added kwargs support for optional fields introduced in MongoDB 2.4
 
         .. versionchanged:: 2.2
            Added support for read only users
@@ -627,8 +644,11 @@ class Database(common.BaseObject):
         """
 
         user = self.system.users.find_one({"user": name}) or {"user": name}
-        user["pwd"] = helpers._password_digest(name, password)
-        user["readOnly"] = common.validate_boolean('read_only', read_only)
+        if password is not None:
+            user["pwd"] = auth._password_digest(name, password)
+        if read_only is not None:
+            user["readOnly"] = common.validate_boolean('read_only', read_only)
+        user.update(kwargs)
 
         try:
             self.system.users.save(user, **self._get_wc_override())
@@ -653,95 +673,88 @@ class Database(common.BaseObject):
         """
         self.system.users.remove({"user": name}, **self._get_wc_override())
 
-    def authenticate(self, name, password):
+    def authenticate(self, name, password=None,
+                     source=None, mechanism='MONGODB-CR', **kwargs):
         """Authenticate to use this database.
 
-        Once authenticated, the user has full read and write access to
-        this database. Raises :class:`TypeError` if either `name` or
-        `password` is not an instance of :class:`basestring`
-        (:class:`str` in python 3). Authentication lasts for the life
-        of the underlying :class:`~pymongo.connection.Connection`, or
-        until :meth:`logout` is called.
+        Authentication lasts for the life of the underlying client
+        instance, or until :meth:`logout` is called.
 
-        The "admin" database is special. Authenticating on "admin"
-        gives access to *all* databases. Effectively, "admin" access
-        means root access to the database.
+        Raises :class:`TypeError` if (required) `name`, (optional) `password`,
+        or (optional) `source` is not an instance of :class:`basestring`
+        (:class:`str` in python 3).
 
         .. note::
-          This method authenticates the current connection, and
-          will also cause all new :class:`~socket.socket` connections
-          in the underlying :class:`~pymongo.connection.Connection` to
-          be authenticated automatically.
+          - This method authenticates the current connection, and
+            will also cause all new :class:`~socket.socket` connections
+            in the underlying client instance to be authenticated automatically.
 
-         - When sharing a :class:`~pymongo.connection.Connection`
-           between multiple threads, all threads will share the
-           authentication. If you need different authentication profiles
-           for different purposes (e.g. admin users) you must use
-           distinct instances of :class:`~pymongo.connection.Connection`.
+          - Authenticating more than once on the same database with different
+            credentials is not supported. You must call :meth:`logout` before
+            authenticating with new credentials.
 
-         - To get authentication to apply immediately to all
-           existing sockets you may need to reset this Connection's
-           sockets using :meth:`~pymongo.connection.Connection.disconnect`.
+          - When sharing a client instance between multiple threads, all
+            threads will share the authentication. If you need different
+            authentication profiles for different purposes you must use
+            distinct client instances.
 
-        .. warning::
-
-          Currently, calls to
-          :meth:`~pymongo.connection.Connection.end_request` will
-          lead to unpredictable behavior in combination with
-          auth. The :class:`~socket.socket` owned by the calling
-          thread will be returned to the pool, so whichever thread
-          uses that :class:`~socket.socket` next will have whatever
-          permissions were granted to the calling thread.
+          - To get authentication to apply immediately to all
+            existing sockets you may need to reset this client instance's
+            sockets using :meth:`~pymongo.mongo_client.MongoClient.disconnect`.
 
         :Parameters:
-          - `name`: the name of the user to authenticate
-          - `password`: the password of the user to authenticate
+          - `name`: the name of the user to authenticate.
+          - `password` (optional): the password of the user to authenticate.
+            Not used with GSSAPI or MONGODB-X509 authentication.
+          - `source` (optional): the database to authenticate on. If not
+            specified the current database is used.
+          - `mechanism` (optional): See
+            :data:`~pymongo.auth.MECHANISMS` for options.
+            Defaults to MONGODB-CR (MongoDB Challenge Response protocol)
+          - `gssapiServiceName` (optional): Used with the GSSAPI mechanism
+            to specify the service name portion of the service principal name.
+            Defaults to 'mongodb'.
+
+        .. versionchanged:: 2.5
+           Added the `source` and `mechanism` parameters. :meth:`authenticate`
+           now raises a subclass of :class:`~pymongo.errors.PyMongoError` if
+           authentication fails due to invalid credentials or configuration
+           issues.
 
         .. mongodoc:: authenticate
         """
         if not isinstance(name, basestring):
             raise TypeError("name must be an instance "
                             "of %s" % (basestring.__name__,))
-        if not isinstance(password, basestring):
+        if password is not None and not isinstance(password, basestring):
             raise TypeError("password must be an instance "
                             "of %s" % (basestring.__name__,))
+        if source is not None and not isinstance(source, basestring):
+            raise TypeError("source must be an instance "
+                            "of %s" % (basestring.__name__,))
+        common.validate_auth_mechanism('mechanism', mechanism)
 
-        # So we can authenticate during a failover. The start_request()
-        # call below will pin the host used for getnonce so we use the
-        # same host for authenticate.
-        read_pref = rp.ReadPreference.PRIMARY_PREFERRED
+        validated_options = {}
+        for option, value in kwargs.iteritems():
+            normalized, val = common.validate_auth_option(option, value)
+            validated_options[normalized] = val
 
-        in_request = self.connection.in_request()
-        try:
-            if not in_request:
-                self.connection.start_request()
-
-            nonce = self.command("getnonce",
-                                 read_preference=read_pref)["nonce"]
-            key = helpers._auth_key(nonce, name, password)
-            try:
-                self.command("authenticate", user=unicode(name),
-                             nonce=nonce, key=key, read_preference=read_pref)
-                self.connection._cache_credentials(self.name,
-                                                   unicode(name),
-                                                   unicode(password))
-                return True
-            except OperationFailure:
-                return False
-        finally:
-            if not in_request:
-                self.connection.end_request()
+        credentials = auth._build_credentials_tuple(mechanism,
+                                source or self.name, unicode(name),
+                                password and unicode(password) or None,
+                                validated_options)
+        self.connection._cache_credentials(self.name, credentials)
+        return True
 
     def logout(self):
-        """Deauthorize use of this database for this connection
-        and future connections.
+        """Deauthorize use of this database for this client instance.
 
         .. note:: Other databases may still be authenticated, and other
            existing :class:`~socket.socket` connections may remain
            authenticated for this database unless you reset all sockets
-           with :meth:`~pymongo.connection.Connection.disconnect`.
+           with :meth:`~pymongo.mongo_client.MongoClient.disconnect`.
         """
-        self.command("logout")
+        # Sockets will be deauthenticated as they are used.
         self.connection._purge_credentials(self.name)
 
     def dereference(self, dbref):
