@@ -16,6 +16,11 @@
 
 from bson.errors import *
 
+try:
+    from ssl import CertificateError
+except ImportError:
+    from pymongo.ssl_match_hostname import CertificateError
+
 
 class PyMongoError(Exception):
     """Base class for all PyMongo exceptions.
@@ -98,8 +103,19 @@ class InvalidURI(ConfigurationError):
     .. versionadded:: 1.5
     """
 
+
 class UnsupportedOption(ConfigurationError):
     """Exception for unsupported options.
 
     .. versionadded:: 2.0
     """
+
+
+class ExceededMaxWaiters(Exception):
+    """Raised when a thread tries to get a connection from a pool and
+    ``max_pool_size * waitQueueMultiple`` threads are already waiting.
+
+    .. versionadded:: 2.6
+    """
+    pass
+
