@@ -34,7 +34,7 @@ try:
     _use_c = True
 except ImportError:
     _use_c = False
-from pymongo.errors import InvalidOperation
+from pymongo.errors import InvalidDocument, InvalidOperation, OperationFailure
 
 
 __ZERO = b("\x00\x00\x00\x00")
@@ -70,8 +70,13 @@ def __pack_message(operation, data):
 def insert(collection_name, docs, check_keys,
            safe, last_error_args, continue_on_error, uuid_subtype):
     """Get an **insert** message.
+
+    .. note:: As of PyMongo 2.6, this function is no longer used. It
+       is being kept (with tests) for backwards compatibility with 3rd
+       party libraries that may currently be using it, but will likely
+       be removed in a future release.
+
     """
-    max_bson_size = 0
     options = 0
     if continue_on_error:
         options += 1
@@ -182,3 +187,68 @@ def kill_cursors(cursor_ids):
     for cursor_id in cursor_ids:
         data += struct.pack("<q", cursor_id)
     return __pack_message(2007, data)
+
+def _do_batched_insert(collection_name, docs, check_keys,
+           safe, last_error_args, continue_on_error, uuid_subtype, client):
+    """Insert `docs` using multiple batches.
+    """
+    def _insert_message(insert_message, send_safe):
+        """Build the insert message with header and GLE.
+        """
+        request_id, final_message = __pack_message(2002, insert_message)
+        if send_safe:
+            request_id, error_message, _ = __last_error(collection_name,
+                                                        last_error_args)
+            final_message += error_message
+        return request_id, final_message
+
+    if not docs:
+        raise InvalidOperation("cannot do an empty bulk insert")
+
+    last_error = None
+    begin = struct.pack("<i", int(continue_on_error))
+    begin += bson._make_c_string(collection_name)
+    message_length = len(begin)
+    data = [begin]
+    for doc in docs:
+        encoded = bson.BSON.encode(doc, check_keys, uuid_subtype)
+        encoded_length = len(encoded)
+        if encoded_length > client.max_bson_size:
+            raise InvalidDocument("BSON document too large (%d bytes)"
+                                  " - the connected server supports"
+                                  " BSON document sizes up to %d"
+                                  " bytes." %
+                                  (encoded_length, client.max_bson_size))
+        message_length += encoded_length
+        if message_length < client.max_message_size:
+            data.append(encoded)
+            continue
+
+        # We have enough data, send this message.
+        send_safe = safe or not continue_on_error
+        try:
+            client._send_message(_insert_message(EMPTY.join(data),
+                                                 send_safe), send_safe)
+        # Exception type could be OperationFailure or a subtype
+        # (e.g. DuplicateKeyError)
+        except OperationFailure, exc:
+            # Like it says, continue on error...
+            if continue_on_error:
+                # Store exception details to re-raise after the final batch.
+                last_error = exc
+            # With unacknowledged writes just return at the first error.
+            elif not safe:
+                return
+            # With acknowledged writes raise immediately.
+            else:
+                raise
+        message_length = len(begin) + encoded_length
+        data = [begin, encoded]
+
+    client._send_message(_insert_message(EMPTY.join(data), safe), safe)
+
+    # Re-raise any exception stored due to continue_on_error
+    if last_error is not None:
+        raise last_error
+if _use_c:
+    _do_batched_insert = _cmessage._do_batched_insert
