@@ -42,13 +42,14 @@ import time
 import warnings
 
 from bson.py3compat import b
-from bson.son import SON
-from pymongo import (common,
+from pymongo import (auth,
+                     common,
                      database,
                      helpers,
                      message,
                      pool,
                      uri_parser)
+from pymongo.common import HAS_SSL
 from pymongo.cursor_manager import CursorManager
 from pymongo.errors import (AutoReconnect,
                             ConfigurationError,
@@ -60,6 +61,7 @@ from pymongo.errors import (AutoReconnect,
 
 EMPTY = b("")
 
+
 def _partition_node(node):
     """Split a host:port string returned from mongod/s into
     a (host, int(port)) pair needed for socket.connect().
@@ -83,11 +85,12 @@ class MongoClient(common.BaseObject):
 
     __max_bson_size = 4 * 1024 * 1024
 
-    def __init__(self, host=None, port=None, max_pool_size=10,
-                 document_class=dict, tz_aware=False, _connect=True, **kwargs):
+    def __init__(self, host=None, port=None, max_pool_size=100,
+                 document_class=dict, tz_aware=False, _connect=True,
+                 **kwargs):
         """Create a new connection to a single MongoDB instance at *host:port*.
 
-        The resultant connection object has connection-pooling built
+        The resultant client object has connection-pooling built
         in. It also performs auto-reconnection when necessary. If an
         operation fails because of a connection error,
         :class:`~pymongo.errors.ConnectionFailure` is raised. If
@@ -118,23 +121,44 @@ class MongoClient(common.BaseObject):
             it must be enclosed in '[' and ']' characters following
             the RFC2732 URL syntax (e.g. '[::1]' for localhost)
           - `port` (optional): port number on which to connect
-          - `max_pool_size` (optional): The maximum number of idle connections
-            to keep open in the pool for future use
+          - `max_pool_size` (optional): The maximum number of connections
+            that the pool will open simultaneously. If this is set, operations
+            will block if there are `max_pool_size` outstanding connections
+            from the pool. Defaults to 100.
           - `document_class` (optional): default class to use for
-            documents returned from queries on this connection
+            documents returned from queries on this client
           - `tz_aware` (optional): if ``True``,
             :class:`~datetime.datetime` instances returned as values
             in a document by this :class:`MongoClient` will be timezone
             aware (otherwise they will be naive)
 
-          **Other optional parameters can be passed as keyword arguments:**
+          | **Other optional parameters can be passed as keyword arguments:**
+
+          - `socketTimeoutMS`: (integer) How long (in milliseconds) a send or
+            receive on a socket can take before timing out.
+          - `connectTimeoutMS`: (integer) How long (in milliseconds) a
+            connection can take to be opened before timing out.
+          - `waitQueueTimeoutMS`: (integer) How long (in milliseconds) a
+            thread will wait for a socket from the pool if the pool has no
+            free sockets.
+          - `waitQueueMultiple`: (integer) Multiplied by max_pool_size to give
+            the number of threads allowed to wait for a socket at one time.
+          - `auto_start_request`: If ``True``, each thread that accesses
+            this :class:`MongoClient` has a socket allocated to it for the
+            thread's lifetime.  This ensures consistent reads, even if you
+            read after an unacknowledged write. Defaults to ``False``
+          - `use_greenlets`: If ``True``, :meth:`start_request()` will ensure
+            that the current greenlet uses the same socket for all
+            operations until :meth:`end_request()`
+
+          | **Write Concern options:**
 
           - `w`: (integer or string) If this is a replica set, write operations
             will block until they have been replicated to the specified number
             or tagged set of servers. `w=<int>` always includes the replica set
             primary (e.g. w=3 means write to the primary and wait until
-            replicated to **two** secondaries). **Passing w=0 disables write
-            acknowledgement and all other write concern options.**
+            replicated to **two** secondaries). Passing w=0 **disables write
+            acknowledgement** and all other write concern options.
           - `wtimeout`: (integer) Used in conjunction with `w`. Specify a value
             in milliseconds to control how long to wait for write propagation
             to complete. If replication does not complete in the given
@@ -144,30 +168,55 @@ class MongoClient(common.BaseObject):
           - `fsync`: If ``True`` force the database to fsync all files before
             returning. When used with `j` the server awaits the next group
             commit before returning.
+
+          | **Replica set keyword arguments for connecting with a replica set
+            - either directly or via a mongos:**
+          | (ignored by standalone mongod instances)
+
           - `replicaSet`: (string) The name of the replica set to connect to.
             The driver will verify that the replica set it connects to matches
             this name. Implies that the hosts specified are a seed list and the
-            driver should attempt to find all members of the set.
-          - `socketTimeoutMS`: (integer) How long (in milliseconds) a send or
-            receive on a socket can take before timing out.
-          - `connectTimeoutMS`: (integer) How long (in milliseconds) a
-            connection can take to be opened before timing out.
+            driver should attempt to find all members of the set. *Ignored by
+            mongos*.
+          - `read_preference`: The read preference for this client. If
+            connecting to a secondary then a read preference mode *other* than
+            PRIMARY is required - otherwise all queries will throw
+            :class:`~pymongo.errors.AutoReconnect` "not master".
+            See :class:`~pymongo.read_preferences.ReadPreference` for all
+            available read preference options.
+          - `tag_sets`: Ignored unless connecting to a replica set via mongos.
+            Specify a priority-order for tag sets, provide a list of
+            tag sets: ``[{'dc': 'ny'}, {'dc': 'la'}, {}]``. A final, empty tag
+            set, ``{}``, means "read from any member that matches the mode,
+            ignoring tags.
+
+          | **SSL configuration:**
+
           - `ssl`: If ``True``, create the connection to the server using SSL.
-          - `read_preference`: The read preference for this connection.
-            See :class:`~pymongo.read_preferences.ReadPreference` for available
-            options.
-          - `auto_start_request`: If ``True``, each thread that accesses
-            this :class:`MongoClient` has a socket allocated to it for the
-            thread's lifetime.  This ensures consistent reads, even if you
-            read after an unacknowledged write. Defaults to ``False``
-          - `use_greenlets`: If ``True``, :meth:`start_request()` will ensure
-            that the current greenlet uses the same socket for all
-            operations until :meth:`end_request()`
+          - `ssl_keyfile`: The private keyfile used to identify the local
+            connection against mongod.  If included with the ``certfile` then
+            only the ``ssl_certfile`` is needed.  Implies ``ssl=True``.
+          - `ssl_certfile`: The certificate file used to identify the local
+            connection against mongod. Implies ``ssl=True``.
+          - `ssl_cert_reqs`: Specifies whether a certificate is required from
+            the other side of the connection, and whether it will be validated
+            if provided. It must be one of the three values ``ssl.CERT_NONE``
+            (certificates ignored), ``ssl.CERT_OPTIONAL``
+            (not required, but validated if provided), or ``ssl.CERT_REQUIRED``
+            (required and validated). If the value of this parameter is not
+            ``ssl.CERT_NONE``, then the ``ssl_ca_certs`` parameter must point
+            to a file of CA certificates. Implies ``ssl=True``.
+          - `ssl_ca_certs`: The ca_certs file contains a set of concatenated
+            "certification authority" certificates, which are used to validate
+            certificates passed from the other end of the connection.
+            Implies ``ssl=True``.
 
         .. seealso:: :meth:`end_request`
 
         .. mongodoc:: connections
 
+        .. versionchanged:: 2.5
+           Added additional ssl options
         .. versionadded:: 2.4
         """
         if host is None:
@@ -182,7 +231,7 @@ class MongoClient(common.BaseObject):
         seeds = set()
         username = None
         password = None
-        db = None
+        self.__default_database_name = None
         opts = {}
         for entity in host:
             if "://" in entity:
@@ -191,7 +240,9 @@ class MongoClient(common.BaseObject):
                     seeds.update(res["nodelist"])
                     username = res["username"] or username
                     password = res["password"] or password
-                    db = res["database"] or db
+                    self.__default_database_name = (
+                        res["database"] or self.__default_database_name)
+
                     opts = res["options"]
                 else:
                     idx = entity.find("://")
@@ -218,8 +269,8 @@ class MongoClient(common.BaseObject):
             options[option] = value
         options.update(opts)
 
-        self.__max_pool_size = common.validate_positive_integer(
-                                                'max_pool_size', max_pool_size)
+        self.__max_pool_size = common.validate_positive_integer_or_none(
+            'max_pool_size', max_pool_size)
 
         self.__cursor_manager = CursorManager(self)
 
@@ -232,8 +283,33 @@ class MongoClient(common.BaseObject):
 
         self.__net_timeout = options.get('sockettimeoutms')
         self.__conn_timeout = options.get('connecttimeoutms')
-        self.__use_ssl = options.get('ssl', False)
-        if self.__use_ssl and not pool.have_ssl:
+        self.__wait_queue_timeout = options.get('waitqueuetimeoutms')
+        self.__wait_queue_multiple = options.get('waitqueuemultiple')
+
+        self.__use_ssl = options.get('ssl', None)
+        self.__ssl_keyfile = options.get('ssl_keyfile', None)
+        self.__ssl_certfile = options.get('ssl_certfile', None)
+        self.__ssl_cert_reqs = options.get('ssl_cert_reqs', None)
+        self.__ssl_ca_certs = options.get('ssl_ca_certs', None)
+
+        ssl_kwarg_keys = [k for k in kwargs.keys() if k.startswith('ssl_')]
+        if self.__use_ssl == False and ssl_kwarg_keys:
+            raise ConfigurationError("ssl has not been enabled but the "
+                                     "following ssl parameters have been set: "
+                                     "%s. Please set `ssl=True` or remove."
+                                     % ', '.join(ssl_kwarg_keys))
+
+        if self.__ssl_cert_reqs and not self.__ssl_ca_certs:
+            raise ConfigurationError("If `ssl_cert_reqs` is not "
+                                     "`ssl.CERT_NONE` then you must "
+                                     "include `ssl_ca_certs` to be able "
+                                     "to validate the server.")
+
+        if ssl_kwarg_keys and self.__use_ssl is None:
+            # ssl options imply ssl = True
+            self.__use_ssl = True
+
+        if self.__use_ssl and not HAS_SSL:
             raise ConfigurationError("The ssl module is not available. If you "
                                      "are using a python version previous to "
                                      "2.6 you must install the ssl package "
@@ -246,7 +322,13 @@ class MongoClient(common.BaseObject):
             self.__net_timeout,
             self.__conn_timeout,
             self.__use_ssl,
-            use_greenlets=self.__use_greenlets)
+            use_greenlets=self.__use_greenlets,
+            ssl_keyfile=self.__ssl_keyfile,
+            ssl_certfile=self.__ssl_certfile,
+            ssl_cert_reqs=self.__ssl_cert_reqs,
+            ssl_ca_certs=self.__ssl_ca_certs,
+            wait_queue_timeout=self.__wait_queue_timeout,
+            wait_queue_multiple=self.__wait_queue_multiple)
 
         self.__document_class = document_class
         self.__tz_aware = common.validate_boolean('tz_aware', tz_aware)
@@ -259,7 +341,8 @@ class MongoClient(common.BaseObject):
         super(MongoClient, self).__init__(**options)
         if self.slave_okay:
             warnings.warn("slave_okay is deprecated. Please "
-                          "use read_preference instead.", DeprecationWarning)
+                          "use read_preference instead.", DeprecationWarning,
+                          stacklevel=2)
 
         if _connect:
             try:
@@ -268,14 +351,22 @@ class MongoClient(common.BaseObject):
                 # ConnectionFailure makes more sense here than AutoReconnect
                 raise ConnectionFailure(str(e))
 
-        if db and username is None:
-            warnings.warn("database name in URI is being ignored. If you wish "
-                          "to authenticate to %s, you must provide a username "
-                          "and password." % (db,))
         if username:
-            db = db or "admin"
-            if not self[db].authenticate(username, password):
-                raise ConfigurationError("authentication failed")
+            mechanism = options.get('authmechanism', 'MONGODB-CR')
+            source = (
+                options.get('authsource')
+                or self.__default_database_name
+                or 'admin')
+
+            credentials = auth._build_credentials_tuple(mechanism,
+                                                        source,
+                                                        unicode(username),
+                                                        unicode(password),
+                                                        options)
+            try:
+                self._cache_credentials(source, credentials, _connect)
+            except OperationFailure, exc:
+                raise ConfigurationError(str(exc))
 
     def _cached(self, dbname, coll, index):
         """Test if `index` is cached.
@@ -330,45 +421,53 @@ class MongoClient(common.BaseObject):
         if index_name in self.__index_cache[database_name][collection_name]:
             del self.__index_cache[database_name][collection_name][index_name]
 
-    def _cache_credentials(self, db_name, username, password):
+    def _cache_credentials(self, source, credentials, connect=True):
         """Add credentials to the database authentication cache
-        for automatic login when a socket is created.
-
-        If credentials are already cached for `db_name` they
-        will be replaced.
+        for automatic login when a socket is created. If `connect` is True,
+        verify the credentials on the server first.
         """
-        self.__auth_credentials[db_name] = (username, password)
+        if source in self.__auth_credentials:
+            # Nothing to do if we already have these credentials.
+            if credentials == self.__auth_credentials[source]:
+                return
+            raise OperationFailure('Another user is already authenticated '
+                                   'to this database. You must logout first.')
 
-    def _purge_credentials(self, db_name=None):
+        if connect:
+            sock_info = self.__socket()
+            try:
+                # Since __check_auth was called in __socket
+                # there is no need to call it here.
+                auth.authenticate(credentials, sock_info, self.__simple_command)
+                sock_info.authset.add(credentials)
+            finally:
+                self.__pool.maybe_return_socket(sock_info)
+
+        self.__auth_credentials[source] = credentials
+
+    def _purge_credentials(self, source):
         """Purge credentials from the database authentication cache.
-
-        If `db_name` is None purge credentials for all databases.
         """
-        if db_name is None:
-            self.__auth_credentials.clear()
-        elif db_name in self.__auth_credentials:
-            del self.__auth_credentials[db_name]
+        if source in self.__auth_credentials:
+            del self.__auth_credentials[source]
 
     def __check_auth(self, sock_info):
         """Authenticate using cached database credentials.
         """
-        authset = sock_info.authset
-        names = set(self.__auth_credentials.iterkeys())
+        if self.__auth_credentials or sock_info.authset:
+            cached = set(self.__auth_credentials.itervalues())
 
-        # Logout from any databases no longer listed in the credentials cache.
-        for dbname in authset - names:
-            try:
-                self.__simple_command(sock_info, dbname, {'logout': 1})
-            # TODO: We used this socket to logout. Fix logout so we don't
-            # have to catch this.
-            except OperationFailure:
-                pass
-            authset.discard(dbname)
+            authset = sock_info.authset.copy()
 
-        for db_name in names - authset:
-            user, pwd = self.__auth_credentials[db_name]
-            self.__auth(sock_info, db_name, user, pwd)
-            authset.add(db_name)
+            # Logout any credentials that no longer exist in the cache.
+            for credentials in authset - cached:
+                self.__simple_command(sock_info, credentials[1], {'logout': 1})
+                sock_info.authset.discard(credentials)
+
+            for credentials in cached - authset:
+                auth.authenticate(credentials,
+                                  sock_info, self.__simple_command)
+                sock_info.authset.add(credentials)
 
     @property
     def host(self):
@@ -390,7 +489,7 @@ class MongoClient(common.BaseObject):
 
     @property
     def is_primary(self):
-        """If this instance is connected to a standalone, a replica-set
+        """If this instance is connected to a standalone, a replica set
         primary, or the master of a master-slave set.
 
         .. versionadded:: 2.3
@@ -407,14 +506,19 @@ class MongoClient(common.BaseObject):
 
     @property
     def max_pool_size(self):
-        """The maximum number of idle connections kept open in the pool for
-        future use.
+        """The maximum number of sockets the pool will open concurrently.
 
-        .. note:: ``max_pool_size`` does not cap the number of concurrent
-          connections to the server; there is currently no way to limit the
-          number of connections. ``max_pool_size`` only limits the number of
-          **idle** connections kept open when they are returned to the pool.
+        When the pool has reached `max_pool_size`, operations block waiting for
+        a socket to be returned to the pool. If ``waitQueueTimeoutMS`` is set,
+        a blocked operation will raise :exc:`~pymongo.errors.ConnectionFailure`
+        after a timeout. By default ``waitQueueTimeoutMS`` is not set.
 
+        .. warning:: SIGNIFICANT BEHAVIOR CHANGE in 2.6. Previously, this
+          parameter would limit only the idle sockets the pool would hold
+          onto, not the number of open sockets. The default has also changed
+          to 100.
+
+        .. versionchanged:: 2.6
         .. versionadded:: 1.11
         """
         return self.__max_pool_size
@@ -454,14 +558,14 @@ class MongoClient(common.BaseObject):
 
     document_class = property(get_document_class, set_document_class,
                               doc="""Default class to use for documents
-                              returned on this connection.
+                              returned from this client.
 
                               .. versionadded:: 1.7
                               """)
 
     @property
     def tz_aware(self):
-        """Does this connection return timezone-aware datetimes?
+        """Does this client return timezone-aware datetimes?
 
         .. versionadded:: 1.8
         """
@@ -476,6 +580,15 @@ class MongoClient(common.BaseObject):
         """
         return self.__max_bson_size
 
+    @property
+    def max_message_size(self):
+        """Return the maximum message size the connected server
+        accepts in bytes.
+
+        .. versionadded:: 2.6
+        """
+        return self.__max_message_size
+
     def __simple_command(self, sock_info, dbname, spec):
         """Send a command to the server.
         """
@@ -494,19 +607,6 @@ class MongoClient(common.BaseObject):
         helpers._check_command_response(response, None, msg)
         return response, end - start
 
-    def __auth(self, sock_info, dbname, user, passwd):
-        """Authenticate socket against database `dbname`.
-        """
-        # Get a nonce
-        response, _ = self.__simple_command(sock_info, dbname, {'getnonce': 1})
-        nonce = response['nonce']
-        key = helpers._auth_key(nonce, user, passwd)
-
-        # Actually authenticate
-        query = SON([('authenticate', 1),
-            ('user', user), ('nonce', nonce), ('key', key)])
-        self.__simple_command(sock_info, dbname, query)
-
     def __try_node(self, node):
         """Try to connect to this node and see if it works for our connection
         type. Returns ((host, port), ismaster, isdbgrid, res_time).
@@ -519,16 +619,22 @@ class MongoClient(common.BaseObject):
 
         # Call 'ismaster' directly so we can get a response time.
         sock_info = self.__socket()
-        response, res_time = self.__simple_command(sock_info,
-                                                   'admin',
-                                                   {'ismaster': 1})
-        self.__pool.maybe_return_socket(sock_info)
+        try:
+            response, res_time = self.__simple_command(sock_info,
+                                                       'admin',
+                                                       {'ismaster': 1})
+        finally:
+            self.__pool.maybe_return_socket(sock_info)
 
         # Are we talking to a mongos?
         isdbgrid = response.get('msg', '') == 'isdbgrid'
 
         if "maxBsonObjectSize" in response:
             self.__max_bson_size = response["maxBsonObjectSize"]
+        if "maxMessageSizeBytes" in response:
+            self.__max_message_size = response["maxMessageSizeBytes"]
+        else:
+            self.__max_message_size = 2 * self.max_bson_size
 
         # Replica Set?
         if not self.__direct:
@@ -619,6 +725,9 @@ class MongoClient(common.BaseObject):
                     raise ConfigurationError("Seed list cannot contain a mix "
                                              "of mongod and mongos instances.")
                 return node
+            except OperationFailure:
+                # The server is available but something failed, probably auth.
+                raise
             except Exception, why:
                 errors.append(str(why))
 
@@ -662,10 +771,20 @@ class MongoClient(common.BaseObject):
                 host_details = "%s:%d:" % (host, port)
             raise AutoReconnect("could not connect to "
                                 "%s %s" % (host_details, str(why)))
-        if self.__auth_credentials:
+        try:
             self.__check_auth(sock_info)
+        except OperationFailure:
+            self.__pool.maybe_return_socket(sock_info)
+            raise
         return sock_info
 
+    def _ensure_connected(self, dummy):
+        """Ensure this client instance is connected to a mongod/s.
+        """
+        host, port = (self.__host, self.__port)
+        if host is None or (port is None and '/' not in host):
+            self.__find_node()
+
     def disconnect(self):
         """Disconnect from MongoDB.
 
@@ -709,7 +828,7 @@ class MongoClient(common.BaseObject):
 
         A more certain way to determine server availability is::
 
-            connection.admin.command('ping')
+            client.admin.command('ping')
 
         .. _select: http://docs.python.org/2/library/select.html#select.select
         """
@@ -719,14 +838,18 @@ class MongoClient(common.BaseObject):
         # calls select() if the socket hasn't been checked in the last second,
         # or it may create a new socket, in which case calling select() is
         # redundant.
+        sock_info = None
         try:
-            sock_info = self.__socket()
-            return not pool._closed(sock_info.sock)
-        except (socket.error, ConnectionFailure):
-            return False
+            try:
+                sock_info = self.__socket()
+                return not pool._closed(sock_info.sock)
+            except (socket.error, ConnectionFailure):
+                return False
+        finally:
+            self.__pool.maybe_return_socket(sock_info)
 
     def set_cursor_manager(self, manager_class):
-        """Set this connection's cursor manager.
+        """Set this client's cursor manager.
 
         Raises :class:`TypeError` if `manager_class` is not a subclass of
         :class:`~pymongo.cursor_manager.CursorManager`. A cursor manager
@@ -741,7 +864,8 @@ class MongoClient(common.BaseObject):
            Deprecated support for external cursor managers.
         """
         warnings.warn("Support for external cursor managers is deprecated "
-                      "and will be removed in PyMongo 3.0.", DeprecationWarning)
+                      "and will be removed in PyMongo 3.0.",
+                      DeprecationWarning, stacklevel=2)
         manager = manager_class(self)
         if not isinstance(manager, CursorManager):
             raise TypeError("manager_class must be a subclass of "
@@ -781,8 +905,8 @@ class MongoClient(common.BaseObject):
                     break
 
         if "code" in details:
-            if details["code"] in [11000, 11001, 12582]:
-                raise DuplicateKeyError(details["err"])
+            if details["code"] in (11000, 11001, 12582):
+                raise DuplicateKeyError(details["err"], details["code"])
             else:
                 raise OperationFailure(details["err"], details["code"])
         else:
@@ -831,29 +955,30 @@ class MongoClient(common.BaseObject):
 
         sock_info = self.__socket()
         try:
-            (request_id, data) = self.__check_bson_size(message)
-            sock_info.sock.sendall(data)
-            # Safe mode. We pack the message together with a lastError
-            # message and send both. We then get the response (to the
-            # lastError) and raise OperationFailure if it is an error
-            # response.
-            rv = None
-            if with_last_error:
-                response = self.__receive_message_on_socket(1, request_id,
-                                                            sock_info)
-                rv = self.__check_response_to_last_error(response)
+            try:
+                (request_id, data) = self.__check_bson_size(message)
+                sock_info.sock.sendall(data)
+                # Safe mode. We pack the message together with a lastError
+                # message and send both. We then get the response (to the
+                # lastError) and raise OperationFailure if it is an error
+                # response.
+                rv = None
+                if with_last_error:
+                    response = self.__receive_message_on_socket(1, request_id,
+                                                                sock_info)
+                    rv = self.__check_response_to_last_error(response)
 
+                return rv
+            except OperationFailure:
+                raise
+            except (ConnectionFailure, socket.error), e:
+                self.disconnect()
+                raise AutoReconnect(str(e))
+            except:
+                sock_info.close()
+                raise
+        finally:
             self.__pool.maybe_return_socket(sock_info)
-            return rv
-        except OperationFailure:
-            self.__pool.maybe_return_socket(sock_info)
-            raise
-        except (ConnectionFailure, socket.error), e:
-            self.disconnect()
-            raise AutoReconnect(str(e))
-        except:
-            sock_info.close()
-            raise
 
     def __receive_data_on_socket(self, length, sock_info):
         """Lowest level receive operation.
@@ -870,16 +995,18 @@ class MongoClient(common.BaseObject):
             message += chunk
         return message
 
-    def __receive_message_on_socket(self, operation, request_id, sock_info):
-        """Receive a message in response to `request_id` on `sock`.
+    def __receive_message_on_socket(self, operation, rqst_id, sock_info):
+        """Receive a message in response to `rqst_id` on `sock`.
 
         Returns the response data with the header removed.
         """
         header = self.__receive_data_on_socket(16, sock_info)
         length = struct.unpack("<i", header[:4])[0]
-        assert request_id == struct.unpack("<i", header[8:12])[0], \
-            "ids don't match %r %r" % (request_id,
-                                       struct.unpack("<i", header[8:12])[0])
+        # No rqst_id for exhaust cursor "getMore".
+        if rqst_id is not None:
+            resp_id = struct.unpack("<i", header[8:12])[0]
+            assert rqst_id == resp_id, "ids don't match %r %r" % (rqst_id,
+                                                                  resp_id)
         assert operation == struct.unpack("<i", header[12:])[0]
 
         return self.__receive_data_on_socket(length - 16, sock_info)
@@ -907,28 +1034,30 @@ class MongoClient(common.BaseObject):
           - `message`: (request_id, data) pair making up the message to send
         """
         sock_info = self.__socket()
-
+        exhaust = kwargs.get('exhaust')
         try:
             try:
-                if "network_timeout" in kwargs:
+                if not exhaust and "network_timeout" in kwargs:
                     sock_info.sock.settimeout(kwargs["network_timeout"])
-                return self.__send_and_receive(message, sock_info)
+                response = self.__send_and_receive(message, sock_info)
+
+                if not exhaust:
+                    if "network_timeout" in kwargs:
+                        sock_info.sock.settimeout(self.__net_timeout)
+
+                return (None, (response, sock_info, self.__pool))
             except (ConnectionFailure, socket.error), e:
                 self.disconnect()
                 raise AutoReconnect(str(e))
         finally:
-            if "network_timeout" in kwargs:
-                try:
-                    # Restore the socket's original timeout and return it to
-                    # the pool
-                    sock_info.sock.settimeout(self.__net_timeout)
-                    self.__pool.maybe_return_socket(sock_info)
-                except socket.error:
-                    # There was an exception and we've closed the socket
-                    pass
-            else:
+            if not exhaust:
                 self.__pool.maybe_return_socket(sock_info)
 
+    def _exhaust_next(self, sock_info):
+        """Used with exhaust cursors to get the next batch off the socket.
+        """
+        return self.__receive_message_on_socket(1, None, sock_info)
+
     def start_request(self):
         """Ensure the current thread or greenlet always uses the same socket
         until it calls :meth:`end_request`. This ensures consistent reads,
@@ -938,10 +1067,10 @@ class MongoClient(common.BaseObject):
         "from __future__ import with_statement", :meth:`start_request` can be
         used as a context manager:
 
-        >>> connection = pymongo.MongoClient(auto_start_request=False)
-        >>> db = connection.test
+        >>> client = pymongo.MongoClient(auto_start_request=False)
+        >>> db = client.test
         >>> _id = db.test_collection.insert({})
-        >>> with connection.start_request():
+        >>> with client.start_request():
         ...     for i in range(100):
         ...         db.test_collection.update({'_id': _id}, {'$set': {'i':i}})
         ...
@@ -1029,13 +1158,10 @@ class MongoClient(common.BaseObject):
 
         Raises :class:`TypeError` if `cursor_id` is not an instance of
         ``(int, long)``. What closing the cursor actually means
-        depends on this connection's cursor manager.
+        depends on this client's cursor manager.
 
         :Parameters:
           - `cursor_id`: id of cursor to close
-
-        .. seealso:: :meth:`set_cursor_manager` and
-           the :mod:`~pymongo.cursor_manager` module
         """
         if not isinstance(cursor_id, (int, long)):
             raise TypeError("cursor_id must be an instance of (int, long)")
@@ -1138,12 +1264,28 @@ class MongoClient(common.BaseObject):
                                            fromhost=from_host)["nonce"]
                 command["username"] = username
                 command["nonce"] = nonce
-                command["key"] = helpers._auth_key(nonce, username, password)
+                command["key"] = auth._auth_key(nonce, username, password)
 
             return self.admin.command("copydb", **command)
         finally:
             self.end_request()
 
+    def get_default_database(self):
+        """Get the database named in the MongoDB connection URI.
+
+        >>> uri = 'mongodb://host/my_database'
+        >>> client = MongoClient(uri)
+        >>> db = client.get_default_database()
+        >>> assert db.name == 'my_database'
+
+        Useful in scripts where you want to choose which database to use
+        based only on the URI in a configuration file.
+        """
+        if self.__default_database_name is None:
+            raise ConfigurationError('No default database defined')
+
+        return self[self.__default_database_name]
+
     @property
     def is_locked(self):
         """Is this server locked? While locked, all write operations
