@@ -15,7 +15,6 @@
 """Utilities for choosing which member of a replica set to read from."""
 
 import random
-from collections import deque
 
 from pymongo.errors import ConfigurationError
 
@@ -82,6 +81,8 @@ _mongos_modes = [
 def mongos_mode(mode):
     return _mongos_modes[mode]
 
+def mongos_enum(enum):
+    return _mongos_modes.index(enum)
 
 def select_primary(members):
     for member in members:
@@ -104,6 +105,10 @@ def select_member_with_tags(members, tags, secondary_only, latency):
         if secondary_only and candidate.is_primary:
             continue
 
+        if not (candidate.is_primary or candidate.is_secondary):
+            # In RECOVERING or similar state
+            continue
+
         if candidate.matches_tags(tags):
             candidates.append(candidate)
 
@@ -143,6 +148,7 @@ def select_member(
         return select_primary(members)
 
     elif mode == PRIMARY_PREFERRED:
+        # Recurse.
         candidate_primary = select_member(members, PRIMARY, [{}], latency)
         if candidate_primary:
             return candidate_primary
@@ -158,6 +164,7 @@ def select_member(
         return None
 
     elif mode == SECONDARY_PREFERRED:
+        # Recurse.
         candidate_secondary = select_member(
             members, SECONDARY, tag_sets, latency)
         if candidate_secondary:
@@ -184,26 +191,21 @@ def select_member(
 secondary_ok_commands = frozenset([
     "group", "aggregate", "collstats", "dbstats", "count", "distinct",
     "geonear", "geosearch", "geowalk", "mapreduce", "getnonce", "authenticate",
+    "text",
 ])
 
 
 class MovingAverage(object):
-    """Tracks a moving average. Not thread-safe.
-    """
-    def __init__(self, window_sz):
-        self.window_sz = window_sz
-        self.samples = deque()
-        self.total = 0
+    def __init__(self, samples):
+        """Immutable structure to track a 5-sample moving average.
+        """
+        self.samples = samples[-5:]
+        assert self.samples
+        self.average = sum(self.samples) / float(len(self.samples))
 
-    def update(self, sample):
-        self.samples.append(sample)
-        self.total += sample
-        if len(self.samples) > self.window_sz:
-            self.total -= self.samples.popleft()
+    def clone_with(self, sample):
+        """Get a copy of this instance plus a new sample"""
+        return MovingAverage(self.samples + [sample])
 
     def get(self):
-        if self.samples:
-            return self.total / float(len(self.samples))
-        else:
-            return None
-
+        return self.average
