@@ -33,7 +33,7 @@ from bson.binary import *
 from bson.py3compat import b, binary_type
 from bson.son import SON
 from nose.plugins.skip import SkipTest
-from test.test_connection import get_connection
+from test.test_client import get_client
 
 
 class TestBinary(unittest.TestCase):
@@ -157,12 +157,12 @@ class TestBinary(unittest.TestCase):
         self.assertEqual(data, encoded)
 
         # Test insert and find
-        conn = get_connection()
-        conn.pymongo_test.drop_collection('java_uuid')
-        coll = conn.pymongo_test.java_uuid
+        client = get_client()
+        client.pymongo_test.drop_collection('java_uuid')
+        coll = client.pymongo_test.java_uuid
         coll.uuid_subtype = JAVA_LEGACY
 
-        coll.insert(docs, safe=True)
+        coll.insert(docs)
         self.assertEqual(5, coll.count())
         for d in coll.find():
             self.assertEqual(d['newguid'], uuid.UUID(d['newguidstring']))
@@ -170,7 +170,7 @@ class TestBinary(unittest.TestCase):
         coll.uuid_subtype = OLD_UUID_SUBTYPE
         for d in coll.find():
             self.assertNotEqual(d['newguid'], d['newguidstring'])
-        conn.pymongo_test.drop_collection('java_uuid')
+        client.pymongo_test.drop_collection('java_uuid')
 
     def test_legacy_csharp_uuid(self):
         if not should_test_uuid:
@@ -229,12 +229,12 @@ class TestBinary(unittest.TestCase):
         self.assertEqual(data, encoded)
 
         # Test insert and find
-        conn = get_connection()
-        conn.pymongo_test.drop_collection('csharp_uuid')
-        coll = conn.pymongo_test.csharp_uuid
+        client = get_client()
+        client.pymongo_test.drop_collection('csharp_uuid')
+        coll = client.pymongo_test.csharp_uuid
         coll.uuid_subtype = CSHARP_LEGACY
 
-        coll.insert(docs, safe=True)
+        coll.insert(docs)
         self.assertEqual(5, coll.count())
         for d in coll.find():
             self.assertEqual(d['newguid'], uuid.UUID(d['newguidstring']))
@@ -242,13 +242,13 @@ class TestBinary(unittest.TestCase):
         coll.uuid_subtype = OLD_UUID_SUBTYPE
         for d in coll.find():
             self.assertNotEqual(d['newguid'], d['newguidstring'])
-        conn.pymongo_test.drop_collection('csharp_uuid')
+        client.pymongo_test.drop_collection('csharp_uuid')
 
     def test_uuid_queries(self):
         if not should_test_uuid:
             raise SkipTest("No uuid module")
 
-        c = get_connection()
+        c = get_client()
         coll = c.pymongo_test.test
         coll.drop()
 
@@ -308,5 +308,6 @@ class TestBinary(unittest.TestCase):
             for proto in xrange(pickle.HIGHEST_PROTOCOL + 1):
                 self.assertEqual(uul, pickle.loads(pickle.dumps(uul, proto)))
 
+
 if __name__ == "__main__":
     unittest.main()
