@@ -27,17 +27,12 @@ from bson.objectid import ObjectId
 from bson.son import SON
 from pymongo.connection import Connection
 from pymongo.mongo_client import MongoClient
-from pymongo.replica_set_connection import ReplicaSetConnection
 from pymongo.mongo_replica_set_client import MongoReplicaSetClient
 from pymongo.errors import ConfigurationError, OperationFailure
+from test import host, port, pair
 from test.utils import drop_collections
 
 
-host = os.environ.get("DB_IP", 'localhost')
-port = int(os.environ.get("DB_PORT", 27017))
-pair = '%s:%d' % (host, port)
-
-
 class TestCommon(unittest.TestCase):
 
     def test_baseobject(self):
@@ -62,6 +57,7 @@ class TestCommon(unittest.TestCase):
 
         warnings.simplefilter("ignore")
 
+        # Connection tests
         c = Connection(pair)
         self.assertFalse(c.slave_okay)
         self.assertFalse(c.safe)
@@ -75,16 +71,61 @@ class TestCommon(unittest.TestCase):
         self.assertFalse(coll.slave_okay)
         self.assertFalse(coll.safe)
         self.assertEqual({}, coll.get_lasterror_options())
+
+        self.assertEqual((False, {}), coll._get_write_mode())
+        coll.safe = False
+        coll.write_concern.update(w=1)
+        self.assertEqual((True, {}), coll._get_write_mode())
+        coll.write_concern.update(w=3)
+        self.assertEqual((True, {'w': 3}), coll._get_write_mode())
+
+        coll.safe = True
+        coll.write_concern.update(w=0)
+        self.assertEqual((False, {}), coll._get_write_mode())
+
+        coll = db.test
+        cursor = coll.find()
+        self.assertFalse(cursor._Cursor__slave_okay)
+        cursor = coll.find(slave_okay=True)
+        self.assertTrue(cursor._Cursor__slave_okay)
+
+        # MongoClient test
+        c = MongoClient(pair)
+        self.assertFalse(c.slave_okay)
+        self.assertTrue(c.safe)
+        self.assertEqual({}, c.get_lasterror_options())
+        db = c.pymongo_test
+        db.drop_collection("test")
+        self.assertFalse(db.slave_okay)
+        self.assertTrue(db.safe)
+        self.assertEqual({}, db.get_lasterror_options())
+        coll = db.test
+        self.assertFalse(coll.slave_okay)
+        self.assertTrue(coll.safe)
+        self.assertEqual({}, coll.get_lasterror_options())
+
+        self.assertEqual((True, {}), coll._get_write_mode())
+        coll.safe = False
+        coll.write_concern.update(w=1)
+        self.assertEqual((True, {}), coll._get_write_mode())
+        coll.write_concern.update(w=3)
+        self.assertEqual((True, {'w': 3}), coll._get_write_mode())
+
+        coll.safe = True
+        coll.write_concern.update(w=0)
+        self.assertEqual((False, {}), coll._get_write_mode())
+
+        coll = db.test
         cursor = coll.find()
         self.assertFalse(cursor._Cursor__slave_okay)
         cursor = coll.find(slave_okay=True)
         self.assertTrue(cursor._Cursor__slave_okay)
 
         # Setting any safe operations overrides explicit safe
-        self.assertTrue(Connection(host, port, wtimeout=1000, safe=False).safe)
+        self.assertTrue(MongoClient(host, port, wtimeout=1000, safe=False).safe)
 
-        c = Connection(pair, slaveok=True, w='majority',
-                                     wtimeout=300, fsync=True, j=True)
+        c = MongoClient(pair, slaveok=True, w='majority',
+                        wtimeout=300, fsync=True, j=True)
         self.assertTrue(c.slave_okay)
         self.assertTrue(c.safe)
         d = {'w': 'majority', 'wtimeout': 300, 'fsync': True, 'j': True}
@@ -102,14 +143,14 @@ class TestCommon(unittest.TestCase):
         cursor = coll.find(slave_okay=False)
         self.assertFalse(cursor._Cursor__slave_okay)
 
-        c = Connection('mongodb://%s/?'
+        c = MongoClient('mongodb://%s/?'
                        'w=2;wtimeoutMS=300;fsync=true;'
                        'journal=true' % (pair,))
         self.assertTrue(c.safe)
         d = {'w': 2, 'wtimeout': 300, 'fsync': True, 'j': True}
         self.assertEqual(d, c.get_lasterror_options())
 
-        c = Connection('mongodb://%s/?'
+        c = MongoClient('mongodb://%s/?'
                        'slaveok=true;w=1;wtimeout=300;'
                        'fsync=true;j=true' % (pair,))
         self.assertTrue(c.slave_okay)
@@ -202,7 +243,7 @@ class TestCommon(unittest.TestCase):
         warnings.resetwarnings()
 
     def test_write_concern(self):
-        c = Connection(pair)
+        c = MongoClient(pair)
 
         self.assertEqual({}, c.write_concern)
         wc = {'w': 2, 'wtimeout': 1000}
@@ -216,7 +257,7 @@ class TestCommon(unittest.TestCase):
         self.assertEqual(wc, c.write_concern)
 
         wc = {'w': 3, 'wtimeout': 1000}
-        c = Connection(w=3, wtimeout=1000)
+        c = MongoClient(pair, w=3, wtimeout=1000)
         self.assertEqual(wc, c.write_concern)
         wc = {'w': 2, 'wtimeout': 1000}
         c.write_concern = wc
@@ -246,39 +287,6 @@ class TestCommon(unittest.TestCase):
             c.write_concern = [('foo', 'bar')]
         self.assertRaises(ConfigurationError, f)
 
-    def test_connection(self):
-        c = Connection(pair)
-        coll = c.pymongo_test.write_concern_test
-        coll.drop()
-        doc = {"_id": ObjectId()}
-        coll.insert(doc)
-        self.assertTrue(coll.insert(doc, safe=False))
-        self.assertTrue(coll.insert(doc, w=0))
-        self.assertTrue(coll.insert(doc))
-        self.assertRaises(OperationFailure, coll.insert, doc, safe=True)
-        self.assertRaises(OperationFailure, coll.insert, doc, w=1)
-
-        c = Connection(pair, safe=True)
-        coll = c.pymongo_test.write_concern_test
-        self.assertTrue(coll.insert(doc, safe=False))
-        self.assertTrue(coll.insert(doc, w=0))
-        self.assertRaises(OperationFailure, coll.insert, doc)
-        self.assertRaises(OperationFailure, coll.insert, doc, safe=True)
-        self.assertRaises(OperationFailure, coll.insert, doc, w=1)
-
-        c = Connection("mongodb://%s/" % (pair,))
-        self.assertFalse(c.safe)
-        coll = c.pymongo_test.write_concern_test
-        self.assertTrue(coll.insert(doc))
-        c = Connection("mongodb://%s/?safe=true" % (pair,))
-        self.assertTrue(c.safe)
-        coll = c.pymongo_test.write_concern_test
-        self.assertRaises(OperationFailure, coll.insert, doc)
-
-        # Equality tests
-        self.assertEqual(c, Connection("mongodb://%s/?safe=true" % (pair,)))
-        self.assertFalse(c != Connection("mongodb://%s/?safe=true" % (pair,)))
-
     def test_mongo_client(self):
         m = MongoClient(pair, w=0)
         coll = m.pymongo_test.write_concern_test
@@ -312,43 +320,8 @@ class TestCommon(unittest.TestCase):
         self.assertEqual(m, MongoClient("mongodb://%s/?w=0" % (pair,)))
         self.assertFalse(m != MongoClient("mongodb://%s/?w=0" % (pair,)))
 
-    def test_replica_set_connection(self):
-        c = Connection(pair)
-        ismaster = c.admin.command('ismaster')
-        if 'setName' in ismaster:
-            setname = str(ismaster.get('setName'))
-        else:
-            raise SkipTest("Not connected to a replica set.")
-        c = ReplicaSetConnection(pair, replicaSet=setname)
-        coll = c.pymongo_test.write_concern_test
-        coll.drop()
-        doc = {"_id": ObjectId()}
-        coll.insert(doc)
-        self.assertTrue(coll.insert(doc, safe=False))
-        self.assertTrue(coll.insert(doc, w=0))
-        self.assertTrue(coll.insert(doc))
-        self.assertRaises(OperationFailure, coll.insert, doc, safe=True)
-        self.assertRaises(OperationFailure, coll.insert, doc, w=1)
-
-        c = ReplicaSetConnection(pair, replicaSet=setname, safe=True)
-        coll = c.pymongo_test.write_concern_test
-        self.assertTrue(coll.insert(doc, safe=False))
-        self.assertTrue(coll.insert(doc, w=0))
-        self.assertRaises(OperationFailure, coll.insert, doc)
-        self.assertRaises(OperationFailure, coll.insert, doc, safe=True)
-        self.assertRaises(OperationFailure, coll.insert, doc, w=1)
-
-        c = ReplicaSetConnection("mongodb://%s/?replicaSet=%s" % (pair, setname))
-        self.assertFalse(c.safe)
-        coll = c.pymongo_test.write_concern_test
-        self.assertTrue(coll.insert(doc))
-        c = ReplicaSetConnection("mongodb://%s/?replicaSet=%s;safe=true" % (pair, setname))
-        self.assertTrue(c.safe)
-        coll = c.pymongo_test.write_concern_test
-        self.assertRaises(OperationFailure, coll.insert, doc)
-
     def test_mongo_replica_set_client(self):
-        c = Connection(pair)
+        c = MongoClient(pair)
         ismaster = c.admin.command('ismaster')
         if 'setName' in ismaster:
             setname = str(ismaster.get('setName'))
