@@ -83,29 +83,27 @@ class TestDBRef(unittest.TestCase):
             self.assertEqual(repr(DBRef("coll", 5, "baz", foo="bar", baz=4)),
                              "DBRef('coll', 5, 'baz', foo='bar', baz=4)")
 
-    def test_cmp(self):
-        self.assertEqual(DBRef("coll", ObjectId("1234567890abcdef12345678")),
-                         DBRef(u"coll", ObjectId("1234567890abcdef12345678")))
-        self.assertNotEqual(DBRef("coll",
-                                  ObjectId("1234567890abcdef12345678")),
-                            DBRef(u"coll",
-                                  ObjectId("1234567890abcdef12345678"), "foo"))
-        self.assertNotEqual(DBRef("coll",
-                                  ObjectId("1234567890abcdef12345678")),
-                            DBRef("col", ObjectId("1234567890abcdef12345678")))
-        self.assertNotEqual(DBRef("coll",
-                            ObjectId("1234567890abcdef12345678")),
+    def test_equality(self):
+        obj_id = ObjectId("1234567890abcdef12345678")
+
+        self.assertEqual(DBRef('foo', 5), DBRef('foo', 5))
+        self.assertEqual(DBRef("coll", obj_id), DBRef(u"coll", obj_id))
+        self.assertNotEqual(DBRef("coll", obj_id),
+                            DBRef(u"coll", obj_id, "foo"))
+        self.assertNotEqual(DBRef("coll", obj_id), DBRef("col", obj_id))
+        self.assertNotEqual(DBRef("coll", obj_id),
                             DBRef("coll", ObjectId(b("123456789011"))))
-        self.assertNotEqual(DBRef("coll",
-                                  ObjectId("1234567890abcdef12345678")), 4)
-        self.assertEqual(DBRef("coll",
-                               ObjectId("1234567890abcdef12345678"), "foo"),
-                         DBRef(u"coll",
-                               ObjectId("1234567890abcdef12345678"), "foo"))
-        self.assertNotEqual(DBRef("coll",
-                                  ObjectId("1234567890abcdef12345678"), "foo"),
-                            DBRef(u"coll",
-                                  ObjectId("1234567890abcdef12345678"), "bar"))
+        self.assertNotEqual(DBRef("coll", obj_id), 4)
+        self.assertEqual(DBRef("coll", obj_id, "foo"),
+                         DBRef(u"coll", obj_id, "foo"))
+        self.assertNotEqual(DBRef("coll", obj_id, "foo"),
+                            DBRef(u"coll", obj_id, "bar"))
+
+        # Explicitly test inequality
+        self.assertFalse(DBRef('foo', 5) != DBRef('foo', 5))
+        self.assertFalse(DBRef("coll", obj_id) != DBRef(u"coll", obj_id))
+        self.assertFalse(DBRef("coll", obj_id, "foo") !=
+                         DBRef(u"coll", obj_id, "foo"))
 
     def test_kwargs(self):
         self.assertEqual(DBRef("coll", 5, foo="bar"),
