@@ -180,7 +180,7 @@ class TestReadPreferences(TestReadPreferencesBase):
         not_used = data_members.difference(used)
         latencies = ', '.join(
             '%s: %dms' % (member.host, member.ping_time.get())
-            for member in c._MongoReplicaSetClient__members.values())
+            for member in c._MongoReplicaSetClient__rs_state.members)
 
         self.assertFalse(not_used,
             "Expected to use primary and all secondaries for mode NEAREST,"
@@ -340,6 +340,26 @@ class TestCommandAndReadPreference(TestReplicaSetClientBase):
                 ('pipeline', [])
             ])))
 
+        # Text search.
+        if version.at_least(self.c, (2, 3, 2)):
+            utils.enable_text_search(self.c)
+            db = self.c.pymongo_test
+
+            # Only way to create an index and wait for all members to build it.
+            index = {
+                'ns': 'pymongo_test.test',
+                'name': 't_text',
+                'key': {'t': 'text'}}
+
+            db.system.indexes.insert(
+                index, manipulate=False, check_keys=False, w=self.w)
+
+            self._test_fn(True, lambda: self.c.pymongo_test.command(SON([
+                ('text', 'test'),
+                ('search', 'foo')])))
+
+            self.c.pymongo_test.test.drop_indexes()
+
     def test_map_reduce_command(self):
         # mapreduce fails if no collection
         self.c.pymongo_test.test.insert({}, w=self.w)
@@ -426,51 +446,24 @@ class TestCommandAndReadPreference(TestReplicaSetClientBase):
 
 
 class TestMovingAverage(unittest.TestCase):
-    def test_empty_moving_average(self):
-        avg = MovingAverage(0)
-        self.assertEqual(None, avg.get())
-        avg.update(10)
-        self.assertEqual(None, avg.get())
+    def test_empty_init(self):
+        self.assertRaises(AssertionError, MovingAverage, [])
 
-    def test_trivial_moving_average(self):
-        avg = MovingAverage(1)
-        self.assertEqual(None, avg.get())
-        avg.update(10)
+    def test_moving_average(self):
+        avg = MovingAverage([10])
         self.assertEqual(10, avg.get())
-        avg.update(20)
-        self.assertEqual(20, avg.get())
-        avg.update(0)
-        self.assertEqual(0, avg.get())
-
-    def test_2_sample_moving_average(self):
-        avg = MovingAverage(2)
-        self.assertEqual(None, avg.get())
-        avg.update(10)
-        self.assertEqual(10, avg.get())
-        avg.update(20)
-        self.assertEqual(15, avg.get())
-        avg.update(30)
-        self.assertEqual(25, avg.get())
-        avg.update(-100)
-        self.assertEqual(-35, avg.get())
-
-    def test_5_sample_moving_average(self):
-        avg = MovingAverage(5)
-        self.assertEqual(None, avg.get())
-        avg.update(10)
-        self.assertEqual(10, avg.get())
-        avg.update(20)
-        self.assertEqual(15, avg.get())
-        avg.update(30)
-        self.assertEqual(20, avg.get())
-        avg.update(-100)
-        self.assertEqual((10 + 20 + 30 - 100) / 4, avg.get())
-        avg.update(17)
-        self.assertEqual((10 + 20 + 30 - 100 + 17) / 5., avg.get())
-        avg.update(43)
-        self.assertEqual((20 + 30 - 100 + 17 + 43) / 5., avg.get())
-        avg.update(-1111)
-        self.assertEqual((30 - 100 + 17 + 43 - 1111) / 5., avg.get())
+        avg2 = avg.clone_with(20)
+        self.assertEqual(15, avg2.get())
+        avg3 = avg2.clone_with(30)
+        self.assertEqual(20, avg3.get())
+        avg4 = avg3.clone_with(-100)
+        self.assertEqual((10 + 20 + 30 - 100) / 4., avg4.get())
+        avg5 = avg4.clone_with(17)
+        self.assertEqual((10 + 20 + 30 - 100 + 17) / 5., avg5.get())
+        avg6 = avg5.clone_with(43)
+        self.assertEqual((20 + 30 - 100 + 17 + 43) / 5., avg6.get())
+        avg7 = avg6.clone_with(-1111)
+        self.assertEqual((30 - 100 + 17 + 43 - 1111) / 5., avg7.get())
 
 
 class TestMongosConnection(unittest.TestCase):
