@@ -22,23 +22,11 @@ from nose.plugins.skip import SkipTest
 
 from test.utils import server_started_with_auth, joinall, RendezvousThread
 from test.test_client import get_client
-from pymongo.mongo_client import MongoClient
-from pymongo.replica_set_connection import MongoReplicaSetClient
+from test.utils import get_pool
 from pymongo.pool import SocketInfo, _closed
 from pymongo.errors import AutoReconnect, OperationFailure
 
 
-def get_pool(client):
-    if isinstance(client, MongoClient):
-        return client._MongoClient__pool
-    elif isinstance(client, MongoReplicaSetClient):
-        writer = client._MongoReplicaSetClient__writer
-        pools = client._MongoReplicaSetClient__members
-        return pools[writer].pool
-    else:
-        raise TypeError(str(client))
-
-
 class AutoAuthenticateThreads(threading.Thread):
 
     def __init__(self, collection, num):
