@@ -335,6 +335,14 @@ class TestURI(unittest.TestCase):
                          parse_uri("mongodb://user%40domain.com"
                                    "@localhost/foo?authMechanism=GSSAPI"))
 
+    def test_parse_uri_unicode(self):
+        # Ensure parsing a unicode returns option names that can be passed
+        # as kwargs. In Python 2.4, keyword argument names must be ASCII.
+        # In all Pythons, str is the type of valid keyword arg names.
+        res = parse_uri(unicode("mongodb://localhost/?fsync=true"))
+        for key in res['options']:
+            self.assertTrue(isinstance(key, str))
+
 
 if __name__ == "__main__":
     unittest.main()
